@@ -1,4 +1,4 @@
-package App::MBUtiny; # $Id: MBUtiny.pm 68 2014-09-16 09:24:10Z abalama $
+package App::MBUtiny; # $Id: MBUtiny.pm 74 2014-09-22 13:12:37Z abalama $
 use strict;
 
 =head1 NAME
@@ -7,7 +7,7 @@ App::MBUtiny - BackUp system for Your WEBsites
 
 =head1 VERSION
 
-Version 1.07
+Version 1.08
 
 =head1 SYNOPSIS
 
@@ -49,6 +49,12 @@ Restore files for all or specified hosts by date
 
 Testing all or specified hosts
 
+=item B<checkup>
+
+    my $status = $mbu->checkup( [qw( ... host names ... )] );
+
+Checking backups for all or specified names of hosts
+
 =item B<c>
 
     my $c = $mbu->c;
@@ -114,7 +120,7 @@ See C<LICENSE> file
 =cut
 
 use vars qw/ $VERSION /;
-$VERSION = '1.07';
+$VERSION = '1.08';
 
 use CTK::Util;
 use CTK::ConfGenUtil;
@@ -183,6 +189,7 @@ sub backup {
     my $c    = $self->c;
     my $config = $c->config;
     my $ret = "";
+    my $status = 1;
     
     #  
     my @tblfields = ( # 
@@ -772,6 +779,7 @@ sub backup {
             }
             
             #     
+            $status = 0 if $ferror;
             $tbl->hr;
             $tbl->row(localtime2date_time, 'STATUS', '', $ferror ? 'ERROR' : 'OK');
             my $tbl_rslt = $tbl->draw() || '';
@@ -800,16 +808,16 @@ sub backup {
                 
                 my $testpfx = $c->testmode() ? '[TEST MODE] ' : '';
                 $ma{'-subject'} ||= !$ferror
-                    ? sprintf($testpfx."MBUtiny %s Report: %s", $VERSION, $hostname)
-                    : sprintf($testpfx."MBUtiny %s ERROR Report: %s", $VERSION, $hostname);
+                    ? sprintf($testpfx."MBUtiny %s BACKUP Report: %s", $VERSION, $hostname)
+                    : sprintf($testpfx."MBUtiny %s BACKUP ERROR Report: %s", $VERSION, $hostname);
                 $ma{'-message'} ||= !$ferror
                     ? sprintf(" \"%s\"   \n\n%s", $hostname, $tbl_rslt)
                     : sprintf(" \"%s\"   \n\n%s", $hostname, $tbl_rslt);
                 $ma{'-message'} .= "\n---\n"
                                  . sprintf("Generated by    : MBUtiny %s\n", $VERSION)
                                  . sprintf("Date generation : %s\n", localtime2date_time())
-                                 . sprintf("MBUTiny Id      : %s\n", '$Id: MBUtiny.pm 68 2014-09-16 09:24:10Z abalama $')
-                                 . sprintf("Time Stamp      : %s\n", CTK::tms())
+                                 . sprintf("MBUTiny Id      : %s\n", '$Id: MBUtiny.pm 74 2014-09-22 13:12:37Z abalama $')
+                                 . sprintf("Time Stamp      : %s\n", $c->tms())
                                  . sprintf("Configuration   : %s\n", $c->cfgfile);
                 $ma{'-attach'} = _attach($ma{'-attach'}) || [];
                 my $sent = send_mail(%ma);
@@ -836,7 +844,7 @@ sub backup {
     $self->msg($ret);
     #$c->log_debug(Data::Dumper::Dumper(@joblist));
     
-    return 1;
+    return $status;
 }
 sub test {
     my $self = shift;
@@ -1080,16 +1088,16 @@ sub test {
 
         my $testpfx = $c->testmode() ? '[TEST MODE] ' : '';
         $ma{'-subject'} ||= $gstatus
-            ? sprintf($testpfx."MBUtiny %s Report", $VERSION)
-            : sprintf($testpfx."MBUtiny %s ERROR Report", $VERSION);
+            ? sprintf($testpfx."MBUtiny %s TEST Report", $VERSION)
+            : sprintf($testpfx."MBUtiny %s TEST ERROR Report", $VERSION);
         $ma{'-message'} ||= $gstatus
             ? sprintf("   \n\n%s", $rslt)
             : sprintf("   \n\n%s", $rslt);
         $ma{'-message'} .= "\n---\n"
             . sprintf("Generated by    : MBUtiny %s\n", $VERSION)
             . sprintf("Date generation : %s\n", localtime2date_time())
-            . sprintf("MBUTiny Id      : %s\n", '$Id: MBUtiny.pm 68 2014-09-16 09:24:10Z abalama $')
-            . sprintf("Time Stamp      : %s\n", CTK::tms())
+            . sprintf("MBUTiny Id      : %s\n", '$Id: MBUtiny.pm 74 2014-09-22 13:12:37Z abalama $')
+            . sprintf("Time Stamp      : %s\n", $c->tms())
             . sprintf("Configuration   : %s\n", $c->cfgfile);
         $ma{'-attach'} = _attach($ma{'-attach'}) || [];
         my $sent = send_mail(%ma);
@@ -1465,6 +1473,211 @@ sub restore {
     
     return 1;
 }
+sub checkup {
+    # CHECKUP:      ,   ,  
+    #          .    , 
+    #    .       mbuchecker.
+    #           
+    #               
+    #  UNKNOWN.             
+    #   .  checkup        X.
+    my $self = shift;
+    my $args = array(shift);
+    my $c    = $self->c;
+    my $config = $c->config;
+    my $status = 1;
+    my @saybuffer;
+    my @hosts; #    ( )
+    my %colls; #    (  ) {URL}{AGENT,MESSAGE}
+    my $sendreport      = value($config => 'sendreport') || 0;
+    my $senderrorreport = value($config => 'senderrorreport') || 0;
+
+    
+    my $tbl_colls = Text::SimpleTable->new(
+            [70, 'URL'],
+            [76, 'MESSAGE'],
+            [8,  'STATUS'], # OK, ERROR
+        );
+    my $tbl_backups = Text::SimpleTable->new(
+            [32, 'HOST'],
+            [24, 'AGENT'],
+            [24, 'SERVER'],
+            [32, 'FILE/SIZE'],
+            [3,  'TYP'],
+            [19, 'DATE'],
+            [8,  'STATUS'], # PASSED, UNKNOWN, FAILED
+        );
+    my $tbl_errors = Text::SimpleTable->new(
+            [160, 'ERROR'],
+        );
+
+    #  
+    my $tdate = pop @$args;
+    my ( $_y, $_m, $_d ) = (localtime( time ))[5,4,3];
+    my @ymd = (($_y+1900), ($_m+1), $_d);
+    if (defined($tdate)) {
+        if ($tdate =~ /(\d{4})\D+(\d{2})\D+(\d{2})/) {
+            @ymd = ($1,$2,$3);
+        } elsif ($tdate =~ /(\d{2})\D+(\d{2})\D+(\d{4})/) {
+            @ymd = ($3,$2,$1);
+        } else {
+            push @$args, $tdate;
+        }
+    }
+    #$c->log_debug(sprintf(">>> %d/%d/%d", @ymd));
+    
+    #  
+    my @joblist = $self->get_jobs;
+    $c->log_debug("Start processing hosts");
+    foreach my $job (sort {(keys(%$a))[0] cmp (keys(%$b))[0]} @joblist) {
+        my $hostname = _name($job);
+        my $hostskip = (!@$args || grep {lc($hostname) eq lc($_)} @$args) ? 0 : 1;
+        $c->log_debug(sprintf("Loading configuration for host \"%s\"... %s", $hostname, ($hostskip ? 'SKIPPED' : 'LOADED') ));
+        next if $hostskip;
+        
+        #  
+        my $enabled  = value($job, $hostname => 'enable');
+        if ($enabled) {
+            $c->log_debug(sprintf("--> Begin processing: \"%s\"", $hostname));
+            push @hosts, $hostname;
+            my $collector_node = _node2anode(node($job, $hostname => 'collector'));
+            foreach my $coll (grep {value($_ => 'uri')}  @$collector_node) {
+                my $coll_uri = value($coll, 'uri') || '';
+                my $agent = new App::MBUtiny::CollectorAgent(
+                            uri         => $coll_uri,
+                            user        => value($coll, 'user'),
+                            password    => value($coll, 'password'),
+                            timeout     => value($coll, 'timeout'),
+                        );
+                my $coll_status = $agent->check;
+                #  
+                $c->log_debug(sprintf(" - %s",$coll_uri));
+                $c->log_error(sprintf("   ERROR: %s",unidecode($agent->error))) unless $coll_status;
+                $colls{$coll_uri} = {
+                        agent => $agent,
+                        message => $coll_status
+                            ? unidecode(value($agent->response => 'data/message') || '')
+                            : unidecode($agent->error)
+                    } unless $colls{$coll_uri};
+            }
+            
+        } else {
+            $c->log_debug(sprintf("--> Skipped \"%s\". Enable flag is off", $hostname));
+        }
+
+    }
+    $c->log_debug("Finish processing hosts");
+    
+    #  ,  
+    my %data;
+    my %req_data;
+    foreach (@hosts) {$req_data{$_} = 0};
+    my @error;
+    $c->log_debug("Start processing collectors");
+    foreach my $coll_uri (keys %colls) {
+        my $agent = $colls{$coll_uri}{agent};
+        $tbl_colls->row($coll_uri, $colls{$coll_uri}{message}, $agent->status ? "OK" : "ERROR");
+        if ($agent->status) {
+            $c->log_debug(sprintf("--> Reading list from collector %s", $coll_uri));
+            #      !
+            my $dte = sprintf("%s.%s.%s", reverse(@ymd));
+            my $stt = $agent->report(
+                    start   => $dte,
+                    finish  => $dte,
+                );
+            $c->log_debug(sprintf("    Status: %s", $stt ? unidecode(value($agent->response => 'data/message') || '') : unidecode($agent->error)));
+            if ($stt) {
+                my $res = _node2anode(node($agent->response => 'data/backup'));
+                $c->log_debug(sprintf("    Loaded: %d backups", fv2zero(value($agent->response => 'data/number'))));
+                #$data_box->loop( agent_name => Data::Dumper::Dumper($res) );
+                foreach my $bu (@$res) {
+                    my $name = uv2null(value($bu => "agent_name"));
+                    $req_data{$name} = 1; #       
+                    $tbl_backups->row(
+                            $name,
+                            sprintf("%s\n%s", uv2null(value($bu => "agent_host")),uv2null(value($bu => "agent_ip"))), 
+                            sprintf("%s\n%s", uv2null(value($bu => "server_host")),uv2null(value($bu => "server_ip"))), 
+                            sprintf("%s\n%s bytes", variant_stf(uv2null(value($bu => "file_name")), 32), correct_number(uv2zero(value($bu => "file_size")))), 
+                            value($bu => "type") ? 'INT' : 'EXT',
+                            uv2null(value($bu => "date_start")), 
+                            value($bu => "status") ? "PASSED" : "FAILED"
+                        );
+                    unless (value($bu => "status")) {
+                        $status = 0;
+                        push @error, sprintf("%s / %s", 
+                                unidecode(uv2null(value($bu => "message"))), 
+                                unidecode(uv2null(value($bu => "comment")))
+                            );
+                    }
+                }
+            }
+
+        }
+    }
+    $tbl_backups->hr if grep { $_ == 0} values(%req_data);
+    foreach my $name (grep {!$req_data{$_}} keys %req_data) {
+        $status = 0;
+        $tbl_backups->row($name,'','','','','',"UNKNOWN");
+    }
+    
+    $tbl_backups->hr;
+    $tbl_backups->row('STATUS', '', '', '', '', localtime2date_time(), $status ? 'PASSED' : 'FAILED');
+    $c->log_debug("Finish processing collectors");
+    
+    # 
+    push @saybuffer, "COLLECTORS:";
+    push @saybuffer, $tbl_colls->draw;
+    push @saybuffer, "BACKUPS:";
+    push @saybuffer, $tbl_backups->draw;
+    if (@error) {
+        push @saybuffer, "ERRORS:";
+        $tbl_errors->row($_) for @error;
+        push @saybuffer, $tbl_errors->draw;
+    }
+    
+    my $result = join("\n",@saybuffer);
+    
+    #          
+    my $maildata = node($config => 'sendmail');
+    if (value($maildata, "to") && ($sendreport || ($senderrorreport && !$status)) ) {
+        my %ma = ();
+        foreach my $k (keys %$maildata) {
+            $ma{"-".$k} = value($maildata, $k);
+        }
+                
+        if ($c->testmode) { #  
+            $ma{'-to'} = value($maildata, 'testmail') || $ma{'-to'} || '';
+            $c->log_debug("Sending report to TEST e-mail");
+        } elsif ($senderrorreport && !$status) { #  !   
+            $ma{'-to'} = value($maildata, 'errormail') || $ma{'-to'} || '';
+            $c->log_debug("Sending report to ERROR e-mail");
+        } else {
+            $c->log_debug("Sending report to e-mail");
+        }
+
+        my $testpfx = $c->testmode() ? '[TEST MODE] ' : '';
+        $ma{'-subject'} ||= $status
+            ? sprintf($testpfx."MBUtiny %s CHECKUP Report", $VERSION)
+            : sprintf($testpfx."MBUtiny %s CHECKUP ERROR Report", $VERSION);
+        $ma{'-message'} ||= $status
+            ? sprintf("        \n\n%s", $result)
+            : sprintf("      \n\n%s", $result);
+        $ma{'-message'} .= "\n---\n"
+            . sprintf("Generated by    : MBUtiny %s\n", $VERSION)
+            . sprintf("Date generation : %s\n", localtime2date_time())
+            . sprintf("MBUTiny Id      : %s\n", '$Id: MBUtiny.pm 74 2014-09-22 13:12:37Z abalama $')
+            . sprintf("Time Stamp      : %s\n", $c->tms())
+            . sprintf("Configuration   : %s\n", $c->cfgfile);
+        $ma{'-attach'} = _attach($ma{'-attach'}) || [];
+        my $sent = send_mail(%ma);
+        $c->log_debug(sprintf(" - %s:", $ma{'-to'}), $sent ? 'OK (Mail has been sent)' : 'FAILED (Mail was not sent)');
+    } else {
+        $c->log_debug("Sending report disabled by user");
+    }    
+    
+    $self->msg($result);
+    return $status;
+}
 
 sub get_jobs { #   .             
     my $self = shift;
